<?php

namespace backend\modules\theme\controllers;

use common\models\CategoryFiles;
use Yii;
use common\models\Category;
use backend\modules\theme\models\search\CategorySearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use common\models\Model;
use yii\helpers\ArrayHelper;

/**
 * CategoryController implements the CRUD actions for Category model.
 */
class CategoryController extends Controller
{
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['post'],
                ],
            ],
        ];
    }

    /**
     * Lists all Category models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new CategorySearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Category model.
     * @param integer $id
     * @return mixed
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Category model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $this->layout = '@backend/modules/theme/views/category/layout.php';

        $model = new Category();
        $model->status = $model::STATUS_ACTIVE;

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('create', [
            'model' => $model,
        ]);
    }

    /**
     * Updates an existing Category model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     */
    public function actionUpdate($id)
    {
        $this->layout = '@backend/modules/theme/views/category/layout.php';

        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
            'model' => $model,
        ]);
    }

    /**
     * Deletes an existing Category model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Files of an existing Category model.
     * If record exists, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     */
    public function actionFiles($id)
    {
        $this->layout = '@backend/modules/theme/views/category/layout.php';

        $categoryModel = $this->findModel($id);
        $models = $categoryModel->categoryFiles;

        if (Yii::$app->request->isPost) {
            $oldIDs = ArrayHelper::map($models, 'id', 'id');
            $models = Model::createMultiple(CategoryFiles::classname(), $models);

            if (Model::loadMultiple($models, Yii::$app->request->post())) {
                $deletedIDs = array_diff($oldIDs, array_filter(ArrayHelper::map($models, 'id', 'id')));

                if (Model::validateMultiple($models)) {
                    $transaction = \Yii::$app->db->beginTransaction();

                    try {
                        $flag = true;

                        if (!empty($deletedIDs)) {
                            CategoryFiles::deleteAll(['id' => $deletedIDs]);
                        }

                        foreach ($models as $model) {
                            $model->category_id = $categoryModel->id;
                            if (!($flag = $model->save(false))) {
                                $transaction->rollBack();
                            }
                        }

                        if ($flag) {
                            $transaction->commit();

                            return $this->redirect(['index']);
                        }
                    } catch (Exception $e) {
                        $transaction->rollBack();
                    }
                }
            }
        }

        return $this->render('file', [
            'categoryModel' => $categoryModel,
            'models' => (empty($models)) ? [new CategoryFiles(['category_id' => $categoryModel->id])] : $models,
        ]);
    }

    /**
     * Finds the Category model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Category the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = Category::findOne($id)) !== null) {
            return $model;
        } else {
            throw new NotFoundHttpException('The requested page does not exist.');
        }
    }

}
